'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = printCallTree;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _util = require('./util');

var _path = require('path');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function printCallTree(_ref) {
  var root = _ref.root,
      compileTimes = _ref.compileTimes,
      maxDepth = _ref.maxDepth,
      config = _ref.config;

  // TODO(vjpr): Sort children by require time?

  (0, _util.printHelp)('Call tree timings', getHelpStr());

  var outTree = {};

  var sortChildren = false;
  var indentSize = 2;
  var threshold = 20;

  // branchGraphic = visual length of indent
  function printNode(node, depth, branchGraphic) {
    if (depth > maxDepth) return;
    var childrenDuration = node.childrenDuration,
        duration = node.duration,
        path = node.path,
        filename = node.filename,
        resolveTime = node.resolveTime,
        childrenResolveTime = node.childrenResolveTime;


    if (path === '<root>') {
      //const str = formatNode()
      //return str
      return '<root>';
    }

    if (!duration) return;
    if (duration < threshold) return;

    var compileTime = _lodash2.default.get(compileTimes[filename], 'duration');
    // Time spent running file excluding require calls and source transforms.
    var nonRequireTime = duration - (childrenDuration + compileTime);

    var str = formatNode();

    return str;

    ////////////////////

    function formatNode() {
      var str = '';
      if (duration) {
        var color = (0, _util.getColorForDuration)(duration);
        str += (0, _util.ms)(duration, color);
      }
      var prettyPath = (0, _path.isAbsolute)(path) ? (0, _path.relative)(process.cwd(), path) : path;
      str += ' ';
      str += prettyPath;
      if (nonRequireTime > 10) str += ' ' + _chalk2.default.yellow.dim('self=') + (0, _util.ms)(nonRequireTime, 'yellow.dim');
      if (compileTime > 10) str += ' ' + (0, _util.ms)(compileTime, 'red.dim', 2);
      //str += ' res: ' + ms(childrenResolveTime, 'white', 2)
      // ---
      if (config.showFullPath) {
        // TODO(vjpr): Can we get the line number of the require stmt?
        var relFilename = (0, _path.relative)(process.cwd(), filename);
        if (config.truncatePaths) {
          var currLineLength = str.length + branchGraphic.length + 3; // 3 = brackets surrounding filename
          relFilename = trimStringToTerminalWidth(currLineLength, relFilename);
        }
        str += _chalk2.default.grey.dim(' (' + relFilename + ')');
      }
      return str;
    }
  }

  ////////////////////

  // This works for traversing our tree!

  // We delete unused nodes otherwise `print-tree` doesn't print correctly.

  var tree = _lodash2.default.cloneDeep(root.model);

  function treePrint(tree, printer) {
    var depth = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

    tree = _lodash2.default.castArray(tree);
    depth++;
    return tree.map(function (node, idx) {
      var str = printer(node, depth, { tree: tree, idx: idx });
      if (!str) return null;
      node.children = node.children && treePrint(node.children, printer, depth);
      return node;
    }).filter(Boolean);
  }
  tree = treePrint(tree, function (node, depth) {
    var str = printNode(node, depth, {});
    if (!str) return null;
    return node;
    //node.pretty = str
    //myPrinter(str, depth)
  });

  //console.log(JSON.stringify(tree, null, 2))

  ////////////////////

  console.log('------');

  var printTree = require('./util/print-tree');
  printTree(tree[0], function (node, branchGraphic, depth) {
    var str = printNode(node, depth, branchGraphic);
    return str;
  }, function (node) {
    return node.children;
  });
}

function getHelpStr() {
  return '\n- Top-down time spent requiring (> 20ms).\n- Yellow shows time spent running self (vs. requiring child modules).';
}

// currStr = string so far
function trimStringToTerminalWidth(currLineLength, lastStringSegment) {
  var targetWidth = process.stdout.columns - currLineLength;
  return truncateToResolvableFullPath(lastStringSegment, targetWidth);
  //return truncateWithEllipses(lastStringSegment, targetWidth)
}

function truncateWithEllipses(text, max) {
  return text.substr(0, max - 1) + (text.length > max ? '…' : '');
}

// So we can iTerm cmd+click still.
// Instead of `foo/bar/ba...`, use `foo/bar`.
// Partial paths don't seem to work in iTerm.
// This approach is good because it allows you to have a link that will get you close enough.
function truncateToResolvableFullPath(text, max) {
  if (text.length > max) {
    var lastSlashLessThanMax = text.substr(0, max).lastIndexOf('/');
    return text.substr(0, lastSlashLessThanMax) + '/…';
  }
  return text;
}

////////////////////////////////////////////////////////////////////////////////

//console.log(outTree)
//console.log(treeify.asTree(outTree), false)

//import treeify from 'treeify'

//root.walk({strategy: 'pre'}, (node) => {
//  const nodePath = node.getPath()
//  const depth = nodePath.length
//  //const depth = node.model.depth
//  printNode(node.model, depth)
// }))

//
//function myPrinter(str, depth) {
//  const indentChars = Array(depth * indentSize).join(' ') + c.grey('| ')
//  str = indent(str, indentChars)
//  console.log(str)
//}

////////////////////

//const treePrinter = require('tree-printer')
//console.log(
//  treePrinter([root.model], {
//    fields: {
//      name: node => {
//        return node.pretty
//      },
//    },
//  }),
//)

////////////////////////////////////////////////////////////////////////////////
//# sourceMappingURL=print-call-tree.js.map