'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = printDupes;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _chalkline = require('chalkline');

var _chalkline2 = _interopRequireDefault(_chalkline);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _util = require('./util');

var _indentString = require('indent-string');

var _indentString2 = _interopRequireDefault(_indentString);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function printDupes(_ref) {
  var requestToFilename = _ref.requestToFilename,
      config = _ref.config;

  (0, _util.printHelp)('Duplicates', getHelpStr());

  (0, _lodash2.default)(requestToFilename).map(function (v, req) {
    if (Object.keys(v).length <= 1) return;
    var requestToFilenames = (0, _lodash2.default)(v).map(function (v, k) {
      var filename = k;
      var _v$node$model = v.node.model,
          cached = _v$node$model.cached,
          duration = _v$node$model.duration,
          childrenDuration = _v$node$model.childrenDuration,
          childrenResolveTime = _v$node$model.childrenResolveTime;

      var parentName = v.node.parent.model.filename;
      return { filename: filename, duration: duration, cached: cached, parentName: parentName, childrenDuration: childrenDuration, childrenResolveTime: childrenResolveTime };
    }).value();

    // Here we can add up all the durations of the cached misses
    //   to reveal how much time can be saved.
    var wastedTime = (0, _lodash2.default)(requestToFilenames)
    //.omitBy('cached')
    //.tail() // We must require it once, so assume the first non-cached call is how long it would take.
    .reduce(function (acc, v) {
      return acc + v.duration;
    }, 0);

    //////////////////////////////////////////////////////////////////////////
    // Render

    var timeStr = (0, _util.ms)(wastedTime, (0, _util.getColorForDuration)(wastedTime));
    console.log(req + ' ' + timeStr);
    (0, _lodash2.default)(requestToFilenames).each(function (_ref2) {
      var filename = _ref2.filename,
          duration = _ref2.duration,
          cached = _ref2.cached,
          parentName = _ref2.parentName,
          childrenDuration = _ref2.childrenDuration;

      var relFilename = _path2.default.relative(process.cwd(), filename);
      var relParentFilename = _path2.default.relative(process.cwd(), parentName);
      var color = cached ? 'green' : 'white';
      var filenameStr = _lodash2.default.get(_chalk2.default, color)(filename) + ' ';
      var totalDuration = duration + childrenDuration;
      filenameStr += (0, _util.ms)(totalDuration, (0, _util.getColorForDuration)(totalDuration));
      var parentStr = _chalk2.default.grey.dim(relParentFilename);
      var cachedStr = cached ? 'hit' : 'miss';
      console.log((0, _indentString2.default)(filenameStr + ' ' + cachedStr, 2));
      if (config.printDupesShowRequester) console.log((0, _indentString2.default)(parentStr, 4));
    });

    //////////////////////////////////////////////////////////////////////////
  }).value();
}

////////////////////////////////////////////////////////////////////////////////

function getHelpStr() {
  return '\n  Show which files made the same request, and whether this request used a cached module from \n  \'require.cache\', or instantiated a fresh module. \n  \n  A cache miss is caused by the same request resolving to a different absolute file path.\n  \n  When using the deduping functionality, a miss is caused by:\n  \n  - the version requested in the package.json file was not satisfied by a root dep, or a pnpm store dep.\n    - FIX: update the deps package.json to a later version.\n  \n  When not using the deduping functionality, a miss is caused by:\n  \n  - an installed module has the dependency in its node_modules folder and will use that copy. (This should never happen with npm3)\n    - FIX: \n      - npm2: npm dedupe\n      - npm3: problem should not occur\n      - enable live-perf deduper\n  \n  Entries are excluded if there was only one request because it would always be a cache miss.\n  \n  Look for dupes that take a long time to require - this can lead to big savings.\n  ';
}

////////////////////////////////////////////////////////////////////////////////

// `E.g.
//
//     ˅ total time spent requiring
// ms 2ms
// ˄ requested package i.e. require('ms') / import 'ms'                      ˅ time spent requiring
//   /xxx/node_modules/.registry.npmjs.org/ms/2.1.1/node_modules/ms/index.js 0ms hit
//   ˄ resolve path                                                              ˄ cache hit/miss
//     node_modules/.registry.npmjs.org/debug/4.1.0/node_modules/debug/src/node.js
//     ˄ requester - file containing require/import statement
//   /xxx/node_modules/.registry.npmjs.org/ms/2.0.0/node_modules/ms/index.js 1ms miss
//     node_modules/.registry.npmjs.org/debug/2.6.9/node_modules/debug/src/debug.js
// `
//# sourceMappingURL=print-dupes.js.map