import {join} from 'path'
import TreeModel from 'tree-model'
import printCallTree from './print-call-tree'
import printDupes from './print-dupes'
import c from 'chalk'
import cl from 'chalkline'

import getCompileTimes from './compile-times'
import getRequireTimes from './require-times'

// TODO(vjpr): Api could be better.
export default function() {

  const ctx = {
    // For finding dupes.
    requestToFilename: {},
    config: {
      printDupesShowRequester: false,
      printDupes: false,
      showFullPath: true,
      truncatePaths: true,
    }
  }

  return {
    start: () => start(ctx),
    end: () => end(ctx),
  }
}

function start(ctx) {
  const tree = new TreeModel()
  const root = tree.parse({path: '<root>', duration: 0, childrenDuration: 0})
  const {requestToFilename} = ctx

  const compileTimes = getCompileTimes()
  getRequireTimes({root, tree, requestToFilename})

  Object.assign(ctx, {root, compileTimes, maxDepth: Infinity})
}

function end(ctx) {
  console.log()
  cl.grey()
  console.log()
  console.log(c.bold('REQUIRE TIMINGS from require-clock'))
  const {root, compileTimes, maxDepth, requestToFilename, config} = ctx
  printCallTree({root, compileTimes, maxDepth, config})
  if (config.printDupes) printDupes({requestToFilename, config})
}
