import _ from 'lodash'
import c from 'chalk'
import {getColorForDuration, ms, printHelp} from './util'
import {isAbsolute, join, relative} from 'path'

export default function printCallTree({root, compileTimes, maxDepth, config}) {
  // TODO(vjpr): Sort children by require time?

  printHelp('Call tree timings', getHelpStr())

  const outTree = {}

  const sortChildren = false
  const indentSize = 2
  const threshold = 20

  // branchGraphic = visual length of indent
  function printNode(node, depth, branchGraphic) {
    if (depth > maxDepth) return
    const {childrenDuration, duration, path, filename, resolveTime, childrenResolveTime} = node

    if (path === '<root>') {
      //const str = formatNode()
      //return str
      return '<root>'
    }

    if (!duration) return
    if (duration < threshold) return

    const compileTime = _.get(compileTimes[filename], 'duration')
    // Time spent running file excluding require calls and source transforms.
    const nonRequireTime = duration - (childrenDuration + compileTime)

    let str = formatNode()

    return str

    ////////////////////

    function formatNode() {
      let str = ''
      if (duration) {
        const color = getColorForDuration(duration)
        str += ms(duration, color)
      }
      let prettyPath = isAbsolute(path) ? relative(process.cwd(), path) : path
      str += ' '
      str += prettyPath
      if (nonRequireTime > 10)
        str += ' ' + c.yellow.dim('self=') + ms(nonRequireTime, 'yellow.dim')
      if (compileTime > 10) str += ' ' + ms(compileTime, 'red.dim', 2)
      //str += ' res: ' + ms(childrenResolveTime, 'white', 2)
      // ---
      if (config.showFullPath) {
        // TODO(vjpr): Can we get the line number of the require stmt?
        let relFilename = relative(process.cwd(), filename)
        if (config.truncatePaths) {
          const currLineLength = str.length + branchGraphic.length + 3 // 3 = brackets surrounding filename
          relFilename = trimStringToTerminalWidth(currLineLength, relFilename)
        }
        str += c.grey.dim(' (' + relFilename + ')')
      }
      return str
    }
  }

  ////////////////////

  // This works for traversing our tree!

  // We delete unused nodes otherwise `print-tree` doesn't print correctly.

  let tree = _.cloneDeep(root.model)

  function treePrint(tree, printer, depth = 0) {
    tree = _.castArray(tree)
    depth++
    return tree.map((node, idx) => {
      const str = printer(node, depth, {tree, idx})
      if (!str) return null
      node.children = node.children && treePrint(node.children, printer, depth)
      return node
    }).filter(Boolean)
  }
  tree = treePrint(tree, (node, depth) => {
    let str = printNode(node, depth, {})
    if (!str) return null
    return node
    //node.pretty = str
    //myPrinter(str, depth)
  })

  //console.log(JSON.stringify(tree, null, 2))

  ////////////////////

  console.log('------')

  const printTree = require('./util/print-tree')
  printTree(
    tree[0],
    (node, branchGraphic, depth) => {
      const str = printNode(node, depth, branchGraphic)
      return str
    },
    node => node.children,
  )

}

function getHelpStr() {
  return `
- Top-down time spent requiring (> 20ms).
- Yellow shows time spent running self (vs. requiring child modules).`
}

// currStr = string so far
function trimStringToTerminalWidth(currLineLength, lastStringSegment) {
  const targetWidth = process.stdout.columns - currLineLength
  return truncateToResolvableFullPath(lastStringSegment, targetWidth)
  //return truncateWithEllipses(lastStringSegment, targetWidth)
}

function truncateWithEllipses(text, max) {
  return text.substr(0, max - 1) + (text.length > max ? '…' : '')
}

// So we can iTerm cmd+click still.
// Instead of `foo/bar/ba...`, use `foo/bar`.
// Partial paths don't seem to work in iTerm.
// This approach is good because it allows you to have a link that will get you close enough.
function truncateToResolvableFullPath(text, max) {
  if (text.length > max) {
    const lastSlashLessThanMax = text.substr(0, max).lastIndexOf('/')
    return text.substr(0, lastSlashLessThanMax) + '/…'
  }
  return text
}

////////////////////////////////////////////////////////////////////////////////

//console.log(outTree)
//console.log(treeify.asTree(outTree), false)

//import treeify from 'treeify'

//root.walk({strategy: 'pre'}, (node) => {
//  const nodePath = node.getPath()
//  const depth = nodePath.length
//  //const depth = node.model.depth
//  printNode(node.model, depth)
// }))

//
//function myPrinter(str, depth) {
//  const indentChars = Array(depth * indentSize).join(' ') + c.grey('| ')
//  str = indent(str, indentChars)
//  console.log(str)
//}

////////////////////

//const treePrinter = require('tree-printer')
//console.log(
//  treePrinter([root.model], {
//    fields: {
//      name: node => {
//        return node.pretty
//      },
//    },
//  }),
//)

////////////////////////////////////////////////////////////////////////////////
