import _ from 'lodash'
import isBuiltinModule from 'is-builtin-module'
import {join} from 'path'
import now from 'performance-now'
import Module from 'module'

export default function({root, tree, requestToFilename}) {
  // You could just re-use the existing tree like require-times does.
  // But, we want to add additional info to the nodes. More convenient I think.

  let current = root

  Module.prototype.requireOrig = Module.prototype.require
  Module.prototype.require = function(request) {
    const parent = this

    let childrenDuration = 0
    let childrenResolveTime = 0
    let childrenCount = 0

    const startTime = now()

    const node = tree.parse({
      path: request,
      startTime,
      childrenDuration,
      childrenResolveTime,
      childrenCount,
    })
    current.addChild(node)
    current = node

    const isMain = false

    const filename = Module._resolveFilename(request, parent, isMain)
    node.model.resolveTime = now() - startTime
    const cachedModule = Module._cache[filename]
    if (cachedModule) node.model.cached = true

    // Needed so we can print dupes.
    if (isModuleRequest(request)) {
      _.set(requestToFilename, [request, filename], {node})
    }

    // Resolves filename, get or set module to cache
    const result = this.requireOrig(request)
    //const result = Module._load(request, this)

    node.model.requester = parent.filename
    node.model.filename = filename
    node.model.endTime = now()
    node.model.duration = node.model.endTime - node.model.startTime

    // Add our duration to parents.
    node.parent.model.childrenCount++
    node.parent.model.childrenDuration += node.model.duration
    node.parent.model.childrenResolveTime += node.model.resolveTime
    current = node.parent

    return result
  }
}

function isModuleRequest(request) {
  return !request.startsWith('.') && !isBuiltinModule(request)
}
